package be.iminds.ilabt.jfed.highlevel.util;

import be.iminds.ilabt.jfed.espec.util.LimitedLiveLog;
import javafx.application.Platform;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.event.Level;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public class InputStreamToLogsThread extends Thread {
    private static final Logger LOG = LoggerFactory.getLogger(InputStreamToLogsThread.class);


    @Nonnull
    private final InputStream inputStream;
    @Nonnull private final String logPrefix;
    @Nullable
    private final LogFunction logFunction;

    @Nullable private Logger log;
    @Nullable private LimitedLiveLog liveLog;

    public InputStreamToLogsThread(@Nonnull InputStream inputStream,
                                   @Nullable Logger log,
                                   @Nullable Level logLevel,
                                   @Nullable String logPrefix,
                                   @Nullable LimitedLiveLog liveLog) {
        this.inputStream = inputStream;
        this.log = log;
        this.liveLog = liveLog;
        this.logPrefix = logPrefix == null ? "" : logPrefix;

        if (logLevel != null && log != null) {
            switch (logLevel) {
                case TRACE:
                    logFunction = log::trace;
                    break;
                case DEBUG:
                    logFunction = log::debug;
                    break;
                case INFO:
                    logFunction = log::info;
                    break;
                case WARN:
                    logFunction = log::warn;
                    break;
                case ERROR:
                    logFunction = log::error;
                    break;
                default:
                    logFunction = null;
            }

        } else
            logFunction = null;
    }

    @Override
    public void run() {
        try {
            BufferedReader stdInput = new BufferedReader(new InputStreamReader(inputStream));

            String s;
            while ((s = stdInput.readLine()) != null) {
                System.out.println(s);
                String str = s;

                if (logFunction != null) {
                    logFunction.log(logPrefix + str);
                }

                if (log != null) {
                    Platform.runLater(() -> liveLog.appendText(str + "\n"));
                }

            }

            inputStream.close();
            liveLog.setEndReached();
        } catch (IOException ex) {
            LOG.error("Error while reading from InputStream to StringProperty", ex);
        }

    }

    private interface LogFunction {
        void log(String text);
    }

   @Nullable
   public LimitedLiveLog getLog() {
        return liveLog;
    }
}
